from onvif import ONVIFCamera
import numpy as np
import requests
from requests.auth import HTTPBasicAuth
from requests.auth import HTTPDigestAuth
from skimage.io import imread
from zeep.transports import Transport

def connection(config):
    try:
        result = ONVIFCamera(config["ip"], config["port"], config["user"], config["password"], "./onvif/",transport=Transport(operation_timeout=10))
        result.devicemgmt.GetServices(False)
        return result
    except: pass

def camera_http_request(uri, config):
    user = config["user"]
    password = config["password"]
    atype = config["atype"]

    if atype == 'digest':
        auth = HTTPDigestAuth(user,password)
    else:
        auth = HTTPBasicAuth(user,password)
    try: 
        response = requests.get(uri,auth=auth, stream = True, timeout=10)
        if response.status_code == 200:
            return response
    except: return None

def get_media(cam):
    if cam:
        try:
            media = cam.create_media_service()
            mediatoken = cam.media.GetProfiles()[0].token
            return media, mediatoken
        except: return None

def get_ptz(cam):
    if cam:
        try:
            ptz = cam.create_ptz_service()
            ptztoken = cam.ptz.GetConfigurations()[0].token
            return ptz, ptztoken
        except: return None

def goto_ptz(x,y,z,ptz,mediat):
    req = ptz.create_type('AbsoluteMove')
    req.ProfileToken = mediat
    req.Position = {'PanTilt': {'x': str(x), 'y': str(y)}, 'Zoom': str(z)}
    req.Speed = {'PanTilt': {'x': 1, 'y': 1}, 'Zoom': 1}
    try:
        resp = ptz.AbsoluteMove(req)
        return True
    except:
        return False
    
def move_ptz(x,y,z,ptz,mediat):
    req = ptz.create_type('RelativeMove')
    req.ProfileToken = mediat
    req.Translation = {'PanTilt': {'x': str(x), 'y': str(y)}, 'Zoom': str(z)}
    req.Speed = {'PanTilt': {'x': 1, 'y': 1}, 'Zoom': 1} 
    try:
        resp = ptz.RelativeMove(req)
        return True
    except:
        return False


def get_position(ptz, mediat):
    try:
        position = ptz.GetStatus(mediat).Position
        x = position.PanTilt.x
        y = position.PanTilt.y
        z = position.Zoom.x
        return float(x), float(y), float(z)
    except:
        return None

def get_snap(media,mediat, config):
    try:
        snapurl = media.GetSnapshotUri(mediat)["Uri"]
        # snapurl = "http://10.105.242.171/cgi-bin/snapshot.cgi?Channel=0"
        response = camera_http_request(snapurl, config)
        if response is not None:
            return imread(response.raw.read(), plugin='imageio')
        else:
            return None
    except: return None

def pan2deg(pos, config):
    mode = config["model"]["position"]["ptype"]
    amin = float(config["model"]["position"]["pan_min"])
    amax = float(config["model"]["position"]["pan_max"])
    if mode == "absolute":
        result = pos * 180
        result = round(result,1)
        return result
    else:
        result = (pos * (amax-amin) / 2) + (amax - (amax-amin)/2 )
        result = round(result,1)
        return result

def deg2pan(pos, config):
    mode = config["model"]["position"]["ptype"]
    amin = float(config["model"]["position"]["pan_min"])
    amax = float(config["model"]["position"]["pan_max"])
    if mode == "absolute":
        result = pos /180
        #Если на выходе угол больше +-180 градусов, то делаю коррекцию вычитая 2пи\
        if result > 1:
            while result > 1:
                result = result - 2
        elif result < -1:
            while result < -1:
                result = result + 2
        return result
    else:
        result = 2 * (pos - (amax - (amax-amin)/2 )) / (amax-amin)
        if result > 1:
            while result > 1:
                result = result - 2
        elif result < -1:
            while result < -1:
                result = result + 2
        return result

def tilt2deg(pos, config):
    mode = config["model"]["position"]["ttype"]
    amin = float(config["model"]["position"]["tilt_min"])
    amax = float(config["model"]["position"]["tilt_max"])
    if mode == "absolute":
        if pos <=0:
            result = -pos * 180
            result = round(result,1)
            return result
        else:
            result = 0;
            return result 
    else:
        result = (pos * (amax-amin) / 2) + (amax - (amax-amin)/2 )
        result = round(result,1)
        return result

def deg2tilt(pos, config):
    mode = config["model"]["position"]["ttype"]
    amin = float(config["model"]["position"]["tilt_min"])
    amax = float(config["model"]["position"]["tilt_max"])
    if mode == "absolute":
        if pos>0:
            result = -pos/180
            if result > 1:
                result = 1
            elif result < -1:
                result = -1
            return result
        else:
            result = 0
    else:
        result = 2 * (pos - (amax - (amax-amin)/2 )) / (amax-amin)
        if result > 1:
            result = 1
        elif result < -1:
            result = -1
        return result
    
def deg2tilt_rel(pos, config):
    amin = float(config["model"]["position"]["tilt_min"])
    amax = float(config["model"]["position"]["tilt_max"])
    space = amax - amin
    return 2 * pos / (space)

def zoom2mm(pos,config):
    mode = config["model"]["focal"]["scale"]
    amin = float(config["model"]["focal"]["min"])
    amax = float(config["model"]["focal"]["max"])
    if mode == "absolute":
        result = pos * amax
        return result
    else:
        result = pos * (amax-amin) + amin
        return result

def mm2zoom(pos,config):
    mode = config["model"]["focal"]["scale"]
    amin = float(config["model"]["focal"]["min"])
    amax = float(config["model"]["focal"]["max"])
    if mode == "absolute":
        result = pos / amax
        if result > 1:
            result = 1
        elif result < 0:
            result = 0
        return result
    else:
        result = (pos - amin) / (amax - amin)
        if result > 1:
            result = 1
        elif result < 0:
            result = 0
        return result
    
def zoom2onvif(pos,config):
    mode = config["model"]["focal"]["scale"]
    amin = float(config["model"]["focal"]["min"])
    amax = float(config["model"]["focal"]["max"])
    space = amax / amin
    result = pos /space
    return result
    
def calc_aov(pos,config):#Угол обзора (от края до края)
    w= 1 / float(config["model"]["resolution"]["width"])
    h= 1 / float(config["model"]["resolution"]["heigth"])
    mtrx_angle = np.arctan(w/h)
    inch = (2.54 * 2/3)
    sizei= 1 / float(config["model"]["resolution"]["matrix_size"])
    sizemm = inch * sizei * 10
    sizew = sizemm * np.cos(mtrx_angle)
    sizeh = sizemm * np.sin(mtrx_angle)
    zx = pos[0]
    zy = pos[1]
    aovw = np.degrees(2*np.arctan(sizew/(2*zx)))
    aovh = np.degrees(2*np.arctan(sizeh/(2*zy)))
    return aovw, aovh

def calc_degshift(x,y,aov, config):
    w= float(config["model"]["resolution"]["width"])
    h= float(config["model"]["resolution"]["heigth"])
    aovx, aovy = aov
    #Найду угол одного пиксела
    onex = aovx / w
    oney = aovy / h
    # ax = aovx * xrel
    # ay = aovy * yrel
    #Получаем значение, на которое нужно сдвинуть камеру с точностью 0.1 градус
    ax = round(2*x*onex,1)
    ay = round(2*y*oney,1)
    # print(x,y)
    # print(ax,ay)
    return ax,ay #Почему перепутано? Хуйня какая-то!

def calc_degshift2(x,y,aov, config):
    w= float(config["model"]["resolution"]["width"])
    h= float(config["model"]["resolution"]["heigth"])
    aovx, aovy = aov
    #Определяю виртуальную высоту треугольника. пикс
    aovx = np.deg2rad(aovx)
    aovy = np.deg2rad(aovy)
    hx = (w/2)/np.tan(aovx)
    hy = (h/2)/np.tan(aovy)
    #Находим тангенсы углов
    tx = x / hx
    ty = y / hy
    #Вычисляю углы
    ax = np.arctan(tx)
    ay = np.arctan(ty)
    ax = np.rad2deg(ax)
    ay = np.rad2deg(ay)


    return ax, ay

# Функция вычисляет AOV из фокусного расстояния по таблице профиля
def lin_aov (focal,profile):
    fd = []
    #Меньше меньшего
    if focal < profile[0][0]:
        fd = profile[0][1],profile[0][2]
    #Больше большего
    elif focal > profile[-1][0]:
        fd = profile[-1][1],profile[-1][2]
    #Равно индексу
    for i in range(len(profile)):
        if profile[i][0] == focal:
            fd = profile[i][1],profile[i][2]
    #Kx+b
    for i in range(len(profile)-1):
        if profile[i][0] < focal < profile[i+1][0]: 
            lf = focal - profile[i][0]
            dy_x = profile[i+1][1] - profile[i][1]
            dy_y = profile[i+1][2] - profile[i][2]
            dx = profile[i+1][0] - profile[i][0]
            aovx = (lf/dx) * dy_x + profile[i][1]
            aovy = (lf/dx) * dy_y + profile[i][2]
            fd = aovx, aovy
    return fd

def ptz_status (ptz, mediat):
    try:
        status = ptz.GetStatus(mediat)
        ptz_status = status.MoveStatus.PanTilt
        zoom_status = status.MoveStatus.Zoom
        idle = True
        if ptz_status!='IDLE' or zoom_status!='IDLE':
            idle = False
        return idle
    except: return True
        

